*Welcome to the Secure Digital club*

Here you will find instructions and code to produce an audio (.wav) player using a SD card, a speaker, 3v coin-cell and an attiny(85) micro-controller. Oh yes, and your favorite tunes!

[Varia Open-Day](http://varia.zone/varia-open-day.html) workshop participants, feel free to skip right ahead to the [Workshop](https://git.vvvvvvaria.org/then/secure-digital-club#workshop) section below.

## Intro
This repository is by no means intended as tutorial/how-to but as a personal archive of notes how to re-create the amazing work by [Elm Chan](http://elm-chan.org/)

It can and will therefore be hyper-specific, incomplete, inconsistent or plain wrong at some points.


## Flashing
To get your micro-controller to do what you want, it needs some instructions. If you have a pre-flashed ATtiny85 micro-controller your can skip to the [Workshop](https://git.vvvvvvaria.org/then/secure-digital-club#workshop)  part of this read-me (below).


#### Prerequisites
* Arduino Uno or Duemilanove (w/ an ATmega328, not an older board with an ATmega168)
* ATtiny45 or ATtiny85 (8-pin DIP package), the attiny85-20pu(2.7-5.5) (20Mhz) is not so load, will test out  attiny85V-10pu(1.8v-5.5) (10Mhz). This means setting the bootloader fuses at 8Mhz and speeding up the soundfiles times two.
* 10 uF capacitor 
* Breadboard
* 8x jumper wires
* Arduino IDE

#### Prepare your Arduino
* Open Arduino IDE, add ATTiny support, [see here](http://highlowtech.org/?p=1695) 
* Load: File>Examples>11.ArduinoISP
* Connect Arduino Uno to computer and select the right board and port from the Tools menu (aka Arduino Uno)
* Upload the sketch to the Arduino Uno
* Disconnect the Arduino Uno


#### Hook up your ATtiny
* Wire up the Arduino to the ATtiny:
  * Pin connections:
    * ATtiny Pin 2 to Arduino Pin 13 (or SCK of another programmer)
	* ATtiny Pin 1 to Arduino Pin 12 (or MISO of another programmer)
	* ATtiny Pin 0 to Arduino Pin 11 (or MOSI of another programmer)
	* ATtiny Reset Pin to Arduino Pin 10 (or RESET of another programmer)

Power the ATtiny with 5v from the Arduino.

For a schematic view, refer to [here](https://git.vvvvvvaria.org/then/secure-digital-club/src/branch/master/wiring-diagrams/arduino-to-attiny-wiring.png) or [here](http://highlowtech.org/wp-content/uploads/2011/06/Screen-shot-2011-06-06-at-1.46.39-PM.png).
ATtiny pin-out [here](https://cdn.sparkfun.com/assets/f/8/f/d/9/52713d5b757b7fc0658b4567.png).

### Burning ATtiny
NOTE: THE FOLLOWING STEPS ARE NEEDED FOR EVERY ATTINY YOU PLAN ON USING WITH THIS PARTICULAR CODE.

#### Burning Bootloader
* Connect Arduino to computer again
* In Arduino IDE select:
	* Tools – Board – ATtiny85 @ 16Mhz (internal oscillator; BOD disabled)
	* Tools – Programmer – Arduino as ISP
	* Tools – Serial Port – that your Arduino is connected to

* Finally: 
	* Tools – Burn Bootloader

To clarify, you are not burning a bootloader here. You are resetting the fuses on the ATtiny to set the internal clock at 16Mhz (it normally runs at 1Mhz). 


#### Getting code on the ATtiny
To go from human-readable (C/C++) instructions to machine code we need to compile. For this task we need a compiler...

NOTE: There are possibly easier ways to do this, like straight from the Arduino IDE.


##### avrdude (bundled with Arduino IDE on OSX)
* On osx, open a Terminal and type: 
``` cd /Applications/Arduino.app/Contents/Java/hardware/tools/avr/bin/ ```

* Press enter
* You're now in the folder with all the avr stuff.

The commands after the dollar sign below are all executed from this folder using the terminal and are followed by the ENTER key.
(so don't type the dollar sign)

```$ ./avrdude ```

This command runs the program avrdude, but without further input just lists all available commands.


##### Create Makefile
To compile code (.c) to binary .hex the avrdude compiler needs instructions as well. For this we need a 'Makefile'. You can create a new one or modify an exiting one. For beginners I suggest using and modifying the Makefile in the Blink example in this repo (code/blink/makefile). Or just trying to get your ATtiny to blink in the first place, as per [this tutorial](http://www.instructables.com/id/Honey-I-Shrunk-the-Arduino-Moving-from-Arduino-t/)!

IMPORTANT: make sure all the paths to files, config files etc are correct! (on osx, avrdude etc is not in your 'path',  or /usr/bin/, so you need full paths to these apps! see the tutorial [here](http://www.instructables.com/id/Honey-I-Shrunk-the-Arduino-Moving-from-Arduino-t/) and source [here](https://github.com/internaut/attiny-instructable))

After you have modified your makefile and written your C code you're ready to compile. To compile, type:

``` $ make ```

This will generate a 'main.hex' file that can be uploaded or 'flashed' to your micro-controller.


##### Flashing HEX and BLINK
In the terminal type:

```$ cd /Applications/Arduino.app/Contents/Java/hardware/tools/avr```


```$ ./bin/avrdude -C etc/avrdude.conf -c avrisp -P /dev/cu.usbmodem1421  -b 19200 -p attiny85 -U flash:w:/Users/your user_name/your/main.hex:i```

Notice the paths.


If all is well your code will be uploaded and running! Time to connect a resistor and led; just need to connect pin 5 (PB0) and/or 6 (PB1) of your ATtiny with an LED respectively (of course you need to add a resistor with at least 150 Ohm before). See [this tutorial](http://www.instructables.com/id/Honey-I-Shrunk-the-Arduino-Moving-from-Arduino-t/) for more info.


##### Play music
The compiled .hex for ATtiny85 at 8MHz can be found in this repository (binaries/tiny.hex). To flash:

``` $ ./bin/avrdude -C etc/avrdude.conf -c avrisp -P /dev/cu.usbmodem1421  -b 19200 -p attiny85 -U flash:w:/Users/your user_name/your/tiny.hex:i```


Now you need to connect the ATtiny to an SD card (reader) or even a SD card mirco SD adaptor. [SpriteTM](http://spritesmods.com/) has an excellent schematic (first one) [here](http://spritesmods.com/?art=doorbell&page=2). Or continue below for a even more minimalistic approach.

Please also refer to the Workshop section below for information on how to format your SD card and the music files.



# Workshop


#### Prerequisites

###### Hardware per unit
* 1x (pre flashed) ATtiny85V-10pu(1.8v-5.5) NOT the attiny85-20pu(2.7-5.5) > consumes to much power (p02.jpg)
* 1x speaker (4, 8, 16 ohm? < under construction) (p01.jpg)
* 1x cr2032 (3v lithium cell) (p03.jpg)
* 1x (micro) SD card (with or without adaptor) (p04.jpg) or (p08.jpg)
* 1x battery holder (lithium cells don't like the heat of your iron and will explode in your face) (p05.jpg)
* music!

Optional
 * female mono jack (p06.jpg)
 * 4k7 resistor (p07.jpg)

###### Software
* FFMPEG
* SOX

#### Formatting sound-files
Place (up to 48 kHz, 8-bit, mono) PCM .wav files on a FAT (32) formatted SD card inside folder called "wav".

NOTE: When flashing the ATtiny at 16Mhz the sound files seem to play a bit fast. You can flash a 8MHz bootloader, but this requires you to speed up the sound files times two ('-speed 2'). This IS required when using an ATtiny85 10pu (runs at max 10Mhz). 

###### Workflow
* Format your Sd card as FAT
* In your terminal: 
  *  ``` $ ffmpeg -i input.mp3  output.wav ``` 
  * ``` $ sox output.wav -r 48000 -c 1 -b 8 putmeonsd.wav ```

To convert a whole folder of files:

``` $ for i in *.mp3; do ffmpeg -i "$i" ${i%.*}.wav; done```

followed by

``` $ for i in *wav; do sox "$i" -r 48000 -c 1 -b 8  ${i%.*}-conv.wav; done ```

Your correctly converted files will have the '-conv.wav' extention. Delete the rest and put all the files ending in '-conv.wav' in a folder called 'wav' on your FAT32 formatted SD card.

FYI: We are still working on a one-liner...

#### Schematic

[SpriteTM](http://spritesmods.com/) has an excellent schematic (first one) [here](http://spritesmods.com/?art=doorbell&page=2). But we can go more minimal (hint: 'forget' the capacitors):


![schematic-view.jpg](images/schematic-view.jpg "schematic-view.jpg")

Schematic drawing of the physical construction:
![physical-view.jpg](images/physical-view.jpg "physical-view.jpg")

For construction on micro SD, see Elm Chan's [schematics](http://elm-chan.org/works/sd8p/rc/sd8p_mo.png)

#### Construction
You're free to come up with a technique to connect the components together as long as you keep the [schematic](#Schematic) in mind (or not ;0). Among popular techniques are: breadboarding, [manhattan](https://hackadaycom.files.wordpress.com/2016/05/0gfwt.jpg?w=400) (great tutorial [here](https://www.parasitstudio.se/building-blog/manhattan-style-pedalbuilding) btw) or [deadbug](https://m.eet.com/media/1180890/thermoml.jpg) circuit design, more crazy circuits [here](https://hackaday.com/2017/09/05/non-standard-circuits-jazz-for-electrons/). Can't decide how to connect the components? Just start soldering up the components pin by pin, either starting from the SD card or the ATtiny. Another approach is to think about what type of device you want to create; will it be a wearable device, a pocket device, a mantle piece? Or what functionality it should have (easy to replace batteries? updatable songs? reset button? skip track button?). Below is an description for a dead-bug style compact unit with replaceable battery, departing from the pins on the SD card in respect to the orientation of the ATtiny.

###### Soldering Tips
* clean the tip of your soldering iron before every usage!
* 'wet' all parts that will come together a.k.a. 'tin' or put solder on all parts before joining them.
* solder will flow to the hottest place ;) and your tip is almost always the hottest…keep this in mind..you can and must make use of this.


###### Steps

1) with pin 1 in the left top corner, bent flat the ATtint85 legs, be careful, these are fragile!

2) place the ATtiny with pin1 in the top left corner on the SD card orientated in such a way that the golden contacts are displayed.

3) Bent the ATtiny pins a little bit so pin 5 touches D0 on the SD card, pin 7 touches the CLK contact of the SD card and pin 8 touches the VCC on the SD card. Lift ATtiny pin 6 up a bit, so it doens't make contact.    ![09](images/09.jpg "09")   ![10](images/10.jpg "10")


4) remember the ATtiny orientation and put it aside.


5) 'wet' or 'tin'  all the SD card contacts with a bit of solder (you can omit the outer two), be careful not to melt the plastic. For easier soldering you can carefully cut the innards of the SD card from the plastic shell! ![11](images/11.jpg "11") ![12](images/12.jpg "12")


6) wet all the legs of the AT tiny with solder, you can omit pin 1.


7) place the ATtiny back in place, put  a TINY bit of solder on your iron and you will be able to 'tack' the ATtiny in place in a breeze. ![13](images/13.jpg "13")

8) remove the battery form the clip if there is one (when heated, the 3v lithium cells tend to explode in your face).


9) wet the legs of the clip. 


10) strip and tin a piece of wire (3.5cm) and connect it to the negative pole of the battery clip.


11) strip and tin a piece of wire (5cm) and connect it to the positive pole of the battery clip. See image ![14](images/14.jpg "14")


12) cut the plastic pins from the battery clip and (hot)glue it to the SD card. ![15](images/15.jpg "15") ![16](images/16.jpg "16")


13) bend the 5cm wire coming from the positive battery lead around the SD card towards the vcc pin of the SD card (and ATtiny ;), cut it to size, wet the end and solder it in place! ![17](images/17.jpg "17") ![18](images/18.jpg "18")


14) the negative pole of the battery must connect to: 
	* the negative pole/gnd of the speaker
	* gnd of the ATtiny (pin 4)
	* gnd of the SD card (at two contacts, use a jumper wire as seen in (step 18))

15) so…from the negative pole of the battery clip, move along to the ATtiny pin 4, solder it down and bend it to the upper most gnd (vss) of the SD card ![19](images/19.jpg "19"). For the speaker use an extra piece of wire (step 19) and use a small piece of jumper wire to connect the two  vss and gnd on the SD card (step 18).

16) cut a piece of wire  (3.5cm) to connect ATtiny pin 2 to SD card pin  D3 (dat 3), cut, strip, tin, solder. ![20](images/20.jpg "20")

17) cut a piece of wire, strip, tin, bent to connect SD cards gnd pin to negative battery clip. ![21](images/21.jpg "21")

18) cut, strip, tin a 2cm wire to connect ATtiny pin 6 to D1 of the SD card, careful not to burn your hands and make sure no wires are touching! ![22](images/22.jpg "22")

19) at this point i solder 4cm of multicore, flexible (in order to have a bit more freedom to position the speaker), wire to the audio out of the ATtiny (pin 3) and the negative pole of the battery for the speaker ![23](images/23.jpg "23")  ![24](images/24.jpg "24")

20)  slip in a battery and you have your boom-box!


![25](images/25.jpg "25")
![26](images/26.jpg "26")
![27](images/27.jpg "27")


#### Notes and features
* Instead of a speaker you can also solder a minijack input or just solder the female jack to the speaker leads and enjoy both!
* Reset playback: connect vss (gnd) to DAT0 to restart playing from the first file (alphabetic order).
* Skip Track: connect vss (gnd) to DAT0 through a 4k7 resistor to skip a track.
* The circuit draws 0.03A the cr2032 lithium battery delivers about 200 mAh, so battery life is 200/3 = 66 hours, almost 3 days of operation time.
 




### FURTHER READING AND CREDITS
http://highlowtech.org/?p=1695 - how to use arduino to flash attiny etc.
 http://elm-chan.org/works/sd8p/report.html  - original creator, source files, great site!
 http://www.instructables.com/id/Attiny85-Sd-Wav-Player/ - compiled code (hex) from here
 http://www.sonsivri.to/forum/index.php?topic=7156.0
 https://www.esp32.com/viewtopic.php?t=2272
 http://spritesmods.com/?art=doorbell&page=2
 http://stefaanlippens.net/audio_conversion_cheat_sheet/
 http://www.instructables.com/id/Attiny85-Sd-Wav-Player/ - tut for raspberry pi as programmer
 https://www.youtube.com/watch?v=9LjfkjwMqXI - diy  programming shield
 https://www.youtube.com/watch?v=BexXvxmOGN8 - another one
 http://highlowtech.org/?p=1695 - programming the chip 
 https://www.youtube.com/watch?v=RVC_nLZy-T8 - smart sdcard microsd adaptor hack:
 https://www.youtube.com/watch?v=tUapZ_JdHLE - interesting audacity tut halfway:
 http://txyzinfo.blogspot.nl/2012/11/AVR-attiny2313-wav-music-player-from-SD-card-microSD-MMC.html - cool
 https://www.youtube.com/watch?v=CDRRwNkzasQ
 http://paja-trb.cz/konstrukce/wav_player.html - pcb version, home etch-able.